local MINOR_VERSION = tonumber(("$Revision: 70346 $"):match("%d+"))
if MINOR_VERSION > Omen.MINOR_VERSION then Omen.MINOR_VERSION = MINOR_VERSION end

local bars
local Threat = LibStub("Threat-2.0")
local GUID = LibStub("LibGUIDRegistry-0.1")
local table_sort = _G.table.sort
local math_abs = _G.math.abs
local OL = LibStub("AceLocale-3.0"):GetLocale("Omen")
local L = LibStub("AceLocale-3.0"):GetLocale("Omen_Overview")
local GetTime = _G.GetTime

local Overview = Omen:NewModule("Overview", Omen.ModuleBase, "AceEvent-3.0", "AceTimer-3.0")
local raidTargets, raidTargetsReverse = {}, {}

local configOptions = {
	RaidIcons = true
}
local options = {
	type = "group",
	name = L["Overview Mode"],
	desc = L["Overview Mode"],
	args = {
		test = {
			type = "execute",
			name = OL["Show test bars"],
			desc = OL["Show test bars"],
			func = function() Omen:EnableModule("Overview"); Overview:Test() end
		},
		raidIcons = {
			type = "toggle",
			name = L["Show raid icons"],
			desc = L["Show raid icons"],
			get = function(info) return Overview:GetOption("RaidIcons") end,
			set = function(info, v)
				Overview:SetOption("RaidIcons", v)
				Overview:ReleaseBars()
			end
		}
	}
}

local function sortBars(a, b)
	if a.value > b.value then
		return true
	elseif a.value == b.value then
		return a.guid > b.guid
	end
	return false
end

function Overview:OnInitialize()
	self:Super("OnInitialize")
	self:RegisterConfigDefaults(configOptions)
	self:RegisterOptions(options)
	self.icon = select(3, GetSpellInfo(32272))
	-- self:RegisterOptions(options)
end

function Overview:Hint()
	return L["Overview Mode\n|cffffffffShows an overview of high-threat raid members|r"]
end

function Overview:UpdateLayout()
	self:ArrangeBars()
end

function Overview:OnEnable()
	self:Super("OnEnable")
	bars = self.bars
	Threat.RegisterCallback(self, "ThreatUpdated")
	Omen:SetTitle(L["Overview Mode"] .. ": " .. UnitName("player"))
	GUID.RegisterCallback(self, "RaidTargetGUIDSet")
end

function Overview:OnDisable()
	self:Super("OnDisable")
	GUID.UnregisterCallback(self, "RaidTargetGUIDSet")
	Threat.UnregisterCallback(self, "ThreatUpdated")
end

function Overview.RaidTargetGUIDSet(lib, guid, name, target, unitID)
	if raidTargetsReverse[target] then
		raidTargets[raidTargetsReverse[target]] = nil
	end
	raidTargets[guid] = target
	raidTargetsReverse[target] = guid
end

local labelsWithIcons = {
	"Player", "LEFT", 27,
	"Icon", "LEFT", 6,
	"Enemy", "LEFT", 47,
	"%Max", "RIGHT", 20
}
local labelsNoIcons = {
	"Player", "LEFT", 30,
	"Enemy", "LEFT", 50,
	"%Max", "RIGHT", 20
}

local lastUpdateTime = 0
function Overview:UpdateBar(srcGuid, dstGuid, threat, overrideMax)
	assert(srcGuid ~= nil)
	assert(dstGuid ~= nil)
	local bar, isNew = self:AcquireBar(srcGuid .. dstGuid)
	if isNew then
		bar:SetLabels( unpack(self:GetOption("RaidIcons") and labelsWithIcons or labelsNoIcons) )
		bar:SetLabel("Enemy", Threat.GUIDNameLookup[dstGuid])
		local pName= Threat.GUIDNameLookup[srcGuid]
		bar:SetLabel("Player", pName or "<unknown>")
		if pName then
			bar:SetClass(select(2, UnitClass(pName)))
		end
	end
	if raidTargets[dstGuid] then
		bar:SetLabel("Icon", raidTargets[dstGuid])
	elseif bar.hasIcon then
		bar:SetLabel("Icon", nil)
	end
	local pct = threat / (overrideMax or Threat:GetMaxThreatOnTarget(dstGuid))
	bar:SetValue(pct)
	if GetTime() - lastUpdateTime > 0.6 then
		lastUpdateTime = GetTime()
		self:ArrangeBars()
	end
end

function Overview:ArrangeBars()
	self:ResetBars()
	table_sort(bars, sortBars)
	for i = 1, 80 do		-- A raid of 40 hunters!
		local bar = bars[i]
		if not bar then
			break
		end
		if bar:GetValue() > 0 and self:AddBar(bar) then
			local pct = bar:GetValue()
			bar:SetPercent(pct)
			bar:SetLabel("%Max", ("%2.1f%%"):format(pct*100))
		else
			bar.frame:Hide()
		end
	end
end

function Overview:ThreatUpdated(event, srcGUID, dstGUID, threat)
	if self.testing then
		self:ReleaseBars()
		self.testing = false
	end
	self:UpdateBar(srcGUID, dstGUID, threat)
end

function Overview:Test()
	self:ReleaseBars()
	self.testing = true

	for i = 1, 25 do
		raidTargets[i] = mod(i,8) + 1
		local temp = string.format("0x%02x", i)
		self:UpdateBar(temp, temp, i, 25)
	end
	self:ArrangeBars()
end
